/*
  LCD routines

    lcd_init()           init
    lcd_fun(char )       write a function
       0x80                 set address to zero
       0x01                 clear display
       0x14                 cursor right 
       0x0e                 display on,cursor on, blink off 
    lcd_wr( char )       write data
    waitms( char )       rough delay routine that's needed

*/

/* lcd control lines */
#define RS 0x40;
#define RW 0x20;
#define E  0x10;


lcd_init(){

   /* change to 8 bit mode so know where we are */
   lcd_fun(0x33);
   PORTB= PORTB | E;  /* send same command again */
   waitms(1);
   PORTB= PORTB ^ E;
   waitms(1);

   lcd_data(0x2);   /* change to 4 bit mode */
   PORTB= PORTB | E;  /* send command, one write as in 8 bit mode */
   waitms(1);
   PORTB= PORTB ^ E;
   waitms(1);

   lcd_fun(1);     /* clear display */
   lcd_fun(0x14);  /* cursor right */
   lcd_fun(0x0e);  /* display on,cursor on, blink off */
   lcd_fun(0x80);

}

lcd_wr(char dat){   /* write to lcd data */

  PORTB= RS;

  lcd_wr2( dat );
    
  waitms(1);

}

lcd_wr2( char dat ){

   lcd_data( dat >> 4 );
   PORTB= PORTB | E;
   waitms(1);
   PORTB= PORTB ^ E;
   waitms(1);

   lcd_data( dat );
   PORTB= PORTB | E;
   waitms(1);
   PORTB= PORTB ^ E;
   waitms(1);

}


lcd_data( char dat ){  /* merge data onto port b */
static char temp;

   temp= PORTB & 0xf0;
   temp|= ( dat & 0xf );
   PORTB= temp;

}

lcd_fun(char dat){   /* load an lcd function */

   PORTB= 0;

   lcd_wr2( dat );

   waitms(2);

}

void waitms( char ms ){  /* wait some msec */

static char t2;

   while( ms ){
      t2= 200;       /* get 5 instructions in loop */
      while( t2 ){
         --t2;
         #asm
         nop
         #endasm
         }
      --ms;
      }

}
