/*
  servo.c

     program to test a servo using the PIC-EL hardware
     RA4 is signal to servo, dit 
     RA3,RA2 are pushbuttons to change the servo position pb2 pb3
     encoder also changes servo position at a higher rate
       still has problems

     (c) 2004  Ron Carr

*/

#include <servo.h>
#include <lcd.h>

#define UP 0x08
#define DN 0x04
#define PULSE 0x10
#define LED1 0x8
#define LED2 0x4
#define LED3 0x2

/* global variables */

char wtemp;  /* for interupt */
char stemp;

char up;       /* flag for a button pushed */
char dn;
char count;
char wcount;   /* working count for ISR */
char mcount;   /* detect displayed count change */
char val[3];   /* count value in ascii */

char step;
char dir;

/* convert to ascii value and store in val[] */
hex2ascii( char dat ){

static char i;

   for( i= 0; i < 3; ++i ) val[i]= 0;

   /* 100 */
   while( dat >= 100 ){
      ++val[0];
      dat-= 100;
      }

   /* 10 */
   while( dat >= 10 ){
      ++val[1];
      dat-= 10;
      }

   val[2]= dat;

   for( i= 0; i < 3; ++i ) val[i]+= '0';

}


/* delay 10 instuctions including call, return and a decfsz-goto */
void tick(){
   /*  1  decfsz
       2  goto
       2  call
       2  return
   */

#asm
    nop
    nop
    nop
#endasm

}


_interupt(){

/* save w and status */
#asm
   movwf   wtemp
   swapf   STATUS,W
   movwf   stemp
#endasm

   INTCON= 0x20;   /* clear interupt flag */
   TMR0= 178;

   wcount= count;

   PORTA= PULSE;
   do{              /* critical timing loop */
      tick();
   }while(--wcount);
   PORTA= 0;

   /* see if want to change the count */
   /* these will change at the interupt rate */
   if( up ) count+= 1;
   if( dn ) count-= 1;
   up= 0;          /* clear button latched result */
   dn= 0;

   count+= step;
   step= 0;

/* restore status */
#asm
   swapf   stemp,W
   movwf   STATUS
   swapf   wtemp,F
   swapf   wtemp,W
#endasm

}  /* retfie sets global interupt */




init(){

   up= 0;
   dn= 0;
   count= 150;
   mcount= 0;
   dir= 0;
   step= 0;

   /* prescaler assigned to tmr0, count is 256 * 78, about 50 hz rate */
   #asm
     clrwdt
   #endasm

   OPTION_REG= 0xc7;
   TRISA= 0x0f;
   TRISB= 0;    /* all outputs */
   TMR0= 178;

   PORTA= 0;
   PORTB= 0xf;

   lcd_init();

   /* allow interupts */
   INTCON= 0xa0;


}

main(){   /* loops via the .h */

static char temp;

   temp= PORTA;
   up|= ( temp & UP ) ^ UP;   /* test switches, latch result */
   dn|= ( temp & DN ) ^ DN;   /* latch cleared in interupt */

   temp= LED1 + LED2 + LED3;  /* display latched switches in LED's */
   if( up ) temp ^= LED2;
   if( dn ) temp ^= LED3;
   PORTB= temp;

     
  /* display the count on lcd */
   if( count != mcount ){
      mcount= count;
      hex2ascii( count );
      lcd_fun(0x80);
      lcd_wr(val[0]);
      lcd_wr('.');
      lcd_wr(val[1]);
      lcd_wr(val[2]);
      lcd_wr(' ');
      lcd_wr('m');
      lcd_wr('s');
      }


   /* encoder ( seems to miss codes when turned fast ) */
   /* decode from click to click */

   temp= PORTA & 3;
   if( temp != 3 && dir == 0){
      dir= ( temp == 1 ) ? UP : DN;
      }

   if( temp == 3 && dir != 0){  /* respond to result */
      step= ( dir == UP ) ? 10 : -10;
      dir= 0;
      }

}



